{+------------------------------------------------------------
 | Unit GenTypedLists1
 |
 | Version: 1.0  Created: 03.06.99 
 |               Last Modified: 03.06.99
 | Author : P. Below
 | Project: Typed lists generator
 | Description:
 |   The typed list generator creates a unit for a list class
 |   from a template and user-supplied information. The default
 |   template is embedded at design-time into the UnitTemplate
 |   container but it can be overridden by specifying a file
 |   on the command line. The default template is typelist_template.pas.
 |   The template contains a number of placeholder strings (called
 |   macros in the source) which have the form <!MACRONAME>. Note
 |   that the macro names are case-sensitive and that no whitespace
 |   is allowed between the macro name and the <! and > delimiters!
 |   To generate the target unit the code scans the template text
 |   for these macro strings and replaces them with other strings.
 |   A number of macros are build-in (see BuildMacroLists method),
 |   the others are listed in a grid for the user to supply values
 |   for. 
 |   The generator can create typed lists for practically any type.
 |   The user has to specify a couple of things so the correct code
 |   in the template is activated. The template code uses conditional
 |   compilation for this, the generator replaces the <!DEFINES>
 |   macro with a list of $DEFINE compiler directives depending on
 |   the state of the three checkboxes on the form. The $DEFINEs
 |   are:
 |     GTL_OWNSOBJECTS: the list contains objects and owns them.
 |                      It will free the objects when the list
 |                      is cleared, freed, or individual items are
 |                      deleted or overwritten.
 |     GTL_USEPOINTERS: the list contains records or other data
 |                      types larger than 4 bytes. The list will
 |                      allocate memory for each item from the
 |                      heap and free this memory when the list
 |                      is cleared, freed, or individual items are
 |                      deleted or overwritten.
 |
 |   If none of these two directives is specified then the list 
 |   is supposed to contain any 4 byte (or smaller) datatype, if
 |   this is a class type then the list does not own the objects
 |   and will do nothing to free objects in the situations listed
 |   above. It acts like TList in this configuration.
 |
 |     GTL_USECOMPAREPROC: Activates a template for a item 
 |                         comparison function. This template
 |                         must be customized for the data type
 |                         the list is to store! A comparison 
 |                         function is needed to Sort the list
 |                         and find items in it.
 | External dependencies:
 |   Custom components:
 |     The generator form uses two custom components: TExPanel
 |     (in expanel.pas) and TTextContainer (in textcontainer.pas).
 |     These two controls need to be installed into the component
 |     palette before the generator form can be opened in the IDE.
 |   Units:
   |   The generated unit depends on the abstracttypedlist.pas
 |     unit that contains the base class of all typed list classes.
 +------------------------------------------------------------}
Unit GenTypedLists1;

Interface

Uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, Expanel, StdCtrls, ExtCtrls, Textcontainer;

Type
  TTypedListgeneratorMainform = Class(TForm)
    UnitTemplate: TTextContainer;
    Panel1: TPanel;
    GenerateButton: TButton;
    CloseButton: TButton;
    Panel2: TPanel;
    OwnsObjectCheckbox: TCheckBox;
    ExPanel1: TExPanel;
    MacroGrid: TStringGrid;
    UsePointersCheckbox: TCheckBox;
    makeCompareProcCheckbox: TCheckBox;
    HelpButton: TButton;
    Procedure FormCreate(Sender: TObject);
    Procedure CloseButtonClick(Sender: TObject);
    Procedure GenerateButtonClick(Sender: TObject);
    Procedure ExPanel1Resize(Sender: TObject);
    Procedure MacroGridDrawCell(Sender: TObject; aCol, aRow: Integer;
      Rect: TRect; State: TGridDrawState);
    Procedure UsePointersCheckboxClick(Sender: TObject);
    Procedure OwnsObjectCheckboxClick(Sender: TObject);
    procedure HelpButtonClick(Sender: TObject);
  Private
    { Private declarations }
    FKnownMacros: TStringlist;
    FUnknownMacros: TStringlist;

    Procedure BuildMacroLists;
    Procedure SizeGrid;
    Procedure BuildUnit( Const unitfilename: String );
    Function  Macro( Const macroname: String ): String;
    Procedure EvaluateCommandline;
    Procedure VerifyMacroInput;
  Public
    { Public declarations }
  End;

Var
  TypedListgeneratorMainform: TTypedListgeneratorMainform;

Implementation

Uses FileCtrl;
{$R *.DFM}

Const
  cDefaultUnitDirectory = 'D:\Daten\Projects\D32';

ResourceString
  cOverwritePrompt = 
    'File %s exists. OK to overwrite?';
  cOneEmptyMacro =
    'Macro %s is undefined. Proceed anyway?';
  cSeveralEmptyMacros =
    'Macros %s are undefined. Proceed anyway?';

  eUnterminatedMacro =
    'Macro delimiter ">" not found in line %s.';
  eUnknownMacro =
    'Unknown macro %s encountered!';
  eFileNotFound =
    'Specified template file %s not found, will use default template.';
  eUnitnameRequired =
    'The UNITNAME cannot be empty.';  

Type
  TMacroProc = Function: String;
  EMacroError = Class( Exception );

{+============================================================
 | Section 1: Auxillary functions 
 +============================================================}

{+------------------------------------------------------------
 | Function IScan
 |
 | Parameters :
 |  ch: Character to scan for
 |  S : String to scan
 |  fromPos: first character to scan
 | Returns    : 
 |  position of next occurence of character ch, or 0, if none 
 |  found
 | Description:
 |  Search for next occurence of a character in a string.
 | Error Conditions: none
 | Created: 27.11.96 by P. Below
 +------------------------------------------------------------}
Function IScan( ch: Char; Const S: String; fromPos: Integer ): Integer;
  Var
    i: Integer;
  Begin
    Result := 0;
    For i := fromPos  To Length(S) Do Begin
      If S[i] = ch Then Begin
        Result := i;
        Break;
      End; { If }
    End; { For }
  End; { IScan }

{ Note: the build-in macros are implemented by value functions
  that return the macros value when called. This is the 
  best way to do it since the value of these macros is
  not known at compile-time, with the exception of VERSION. 
  The value functions must be of type TMacroProc.  The following
  three functions server as value functions for the three build-in
  macros. }

{ Returns the string for the VERSION macro. }
Function MacroVersion:  String;
  Begin 
    Result := '1.0'; 
  End; { MacroVersion } 

{ Returns the string for the DATE macro. }
Function MacroDate: String;
  Begin
    Result := DateToStr( Date );
  End; { MacroData }

{ Returns the string for the DEFINES macro. The $DEFINE directives
  are assembled according to the state of the three checkboxes on 
  the main form.}
Function MacroDefines: String;
  Procedure Add( Var R: String; Const S: String ); 
    Const
      CRLF = #13#10;
    Begin
      If Length(R) > 0 Then
        R := R + CRLF;
      R := R + S;
    End; { Add }
  Begin
    Result := EmptyStr;
    If TypedListgeneratorMainform.OwnsObjectCheckbox.Checked Then
      Add( Result, '{$DEFINE GTL_OWNSOBJECTS}' );

    If TypedListgeneratorMainform.UsePointersCheckbox.Checked Then
      Add( Result, '{$DEFINE GTL_USEPOINTERS}' );

    If TypedListgeneratorMainform.makeCompareProcCheckbox.Checked Then
      Add( Result, '{$DEFINE GTL_USECOMPAREPROC}' );
  End; { MacroDefines }

{+============================================================
 | Section 2: Methods of TTypedListgeneratorMainform.
 |    Part 1. Form event handlers, Constructors and Destructors
 +============================================================}

{+------------------------------------------------------------
 | Procedure TTypedListgeneratorMainform.FormCreate
 |
 | Event      : OnCreate
 | Used by    : the form
 | Call method: static
 | Visibility : published
 | Description:
 |   Creates the two stringlist objects used to hold macro names,
 |   evaluates a command line, if present, fills the macro lists,
 |   and sizes the columns in the input grid. If we have a 
 |   helpfile in the apps directory it is assigned to the 
 |   Application.Helpfile property.
 | Error Conditions: none
 | Created: 03.06.99 by P. Below
 +------------------------------------------------------------}
Procedure TTypedListgeneratorMainform.FormCreate(Sender: TObject);
  Var
    filename: String;
  Begin
    FKnownMacros:= TStringlist.Create;
    FUnknownMacros:= TStringlist.Create;
    EvaluateCommandline;
    BuildMacrolists;
    SizeGrid;

    filename := ChangeFileExt( Application.Exename, '.HLP' );
    If FileExists( filename ) Then 
      Application.Helpfile := filename
    Else
      HelpButton.Enabled := False;
  End; { TTypedListgeneratorMainform.FormCreate }

{+============================================================
 | Section 2: Methods of TTypedListgeneratorMainform.
 |    Part 2. Methods that are triggered in response to user
 |            action
 |        2a. Button click handlers
 +============================================================}

{+------------------------------------------------------------
 | Procedure TTypedListgeneratorMainform.CloseButtonClick
 |
 | Event      : OnClick
 | Used by    : CloseButton
 | Call method: static
 | Visibility : published
 | Description:
 |   Closes the form. There are no preconditions for the form 
 |   to allow a close.
 | Error Conditions: none
 | Created: 03.06.99 by P. Below
 +------------------------------------------------------------}
Procedure TTypedListgeneratorMainform.CloseButtonClick(Sender: TObject);
  Begin
    Close;
  End; {  TTypedListgeneratorMainform.CloseButtonClick  }

{+------------------------------------------------------------
 | Procedure TTypedListgeneratorMainform.GenerateButtonClick
 |
 | Event      : OnClick
 | Used by    : GenerateButton
 | Call method: static
 | Visibility : published
 | Description:
 |   We first ask the user to select a directory where the generated
 |   unit is to be placed and then start the build process. The unit
 |   filename is the unit name with .PAS extension. If the file already
 |   exists the user is asked if it is OK to overwrite. He can cancel
 |   the build at this point.
 |   The generate button is always enabled since we cannot determine 
 |   up front if one of the macros can be left empty or not, without
 |   loosing generality or complicating the macro syntax. The first
 |   thing the method does is call a validation method that will ask
 |   the user to confirm that any macros left blank are OK to use
 |   in this state. The effect of a blank macro will be that nothing
 |   of it remains in the produced unit. This is OK for a macro like
 |   USESCLAUSE but would be fatal for one like UNITNAME!
 | Error Conditions: none
 | Created: 03.06.99 by P. Below
 +------------------------------------------------------------}
Procedure TTypedListgeneratorMainform.GenerateButtonClick(Sender: TObject);
  Var
    unitfile: String;
  Begin
    VerifyMacroInput;

    If DirectoryExists( cDefaultUnitDirectory ) Then 
      unitfile := cDefaultUnitDirectory
    Else
      unitfile := EmptyStr;

    If SelectDirectory( unitfile, [ sdAllowCreate, sdPerformCreate ], 0 )
    Then Begin
      unitfile := unitfile + '\' + Macro( 'UNITNAME' ) + '.PAS';
      If FileExists( unitfile ) Then Begin
        If MessageDlg(
             Format( cOverwritePrompt, [ unitfile ] ),
             mtConfirmation, mbYesNoCancel, 0 
                     ) <> mrYes 
        Then 
          Exit;
      End; { If }
    End; { If }
    BuildUnit( unitfile );
  End; { TTypedListgeneratorMainform.GenerateButtonClick }

{+------------------------------------------------------------
 | Procedure TTypedListgeneratorMainform.HelpButtonClick
 |
 | Event      : OnClick
 | Used by    : HelpButton
 | Call method: static
 | Visibility : published
 | Description:
 |   Calls up the helpfiles content page.
 | Error Conditions: none
 | Created: 03.06.99 by P. Below
 +------------------------------------------------------------}
Procedure TTypedListgeneratorMainform.HelpButtonClick(Sender: TObject);
  Begin
    Application.HelpCommand( HELP_CONTENTS, 0 );
  End; { TTypedListgeneratorMainform.HelpButtonClick }

{+============================================================
 | Section 2: Methods of TTypedListgeneratorMainform.
 |        2b. Menu click handlers
 |            none, the form does not have any menus.
 +============================================================}
{+============================================================
 | Section 2: Methods of TTypedListgeneratorMainform
 |    Part 2. Methods that are triggered in response to user
 |            action
 |        2c. Other control event handlers
 +============================================================}

{+------------------------------------------------------------
 | Procedure TTypedListgeneratorMainform.ExPanel1Resize
 |
 | Event      : OnResize
 | Used by    : ExPanel1
 | Call method: static
 | Visibility : published
 | Description:
 |   We adjust the size of the right column of the MacroGrid here.
 |   The grid is client-aligned to the panel.
 | Error Conditions: none
 | Created: 03.06.99 by P. Below
 +------------------------------------------------------------}
Procedure TTypedListgeneratorMainform.ExPanel1Resize(Sender: TObject);
  Begin
    With MacroGrid Do
      ColWidths[ 1 ] := ClientWidth - ColWidths[0] - 5;
  End; { TTypedListgeneratorMainform.ExPanel1Resize }

{+------------------------------------------------------------
 | Procedure TTypedListgeneratorMainform.UsePointersCheckboxClick
 |
 | Event      : OnClick
 | Used by    : UsePointersCheckbox
 | Call method: static
 | Visibility : published
 | Description:
 |   The two checkboxes UsePointersCheckbox and OwnsObjectCheckbox
 |   are mutually exclusive, only one can be checked. But both can
 |   be unchecked.
 | Error Conditions: none
 | Created: 03.06.99 by P. Below
 +------------------------------------------------------------}
Procedure TTypedListgeneratorMainform.UsePointersCheckboxClick(
    Sender: TObject);
  Begin
    If UsePointersCheckbox.Checked Then
      OwnsObjectCheckbox.Checked := False;
  End; { TTypedListgeneratorMainform.UsePointersCheckboxClick }

{+------------------------------------------------------------
 | Procedure TTypedListgeneratorMainform.OwnsObjectCheckboxClick
 |
 | Event      : OnClick
 | Used by    : OwnsObjectCheckbox
 | Call method: static
 | Visibility : published
 | Description:
 |   The two checkboxes UsePointersCheckbox and OwnsObjectCheckbox
 |   are mutually exclusive, only one can be checked. But both can
 |   be unchecked.
 | Error Conditions: none
 | Created: 03.06.99 by P. Below
 +------------------------------------------------------------}
Procedure TTypedListgeneratorMainform.OwnsObjectCheckboxClick(
    Sender: TObject);
  Begin
    If OwnsObjectCheckbox.Checked Then
      UsePointersCheckbox.Checked := False;
  End; { TTypedListgeneratorMainform.OwnsObjectCheckboxClick }

{+============================================================
 | Section 2: Methods of TTypedListgeneratorMainform
 |    Part 3. Handlers For Windows messages
 |            none
 +============================================================}
{+============================================================
 | Section 2: Methods of TTypedListgeneratorMainform
 |    Part 4. Other event handlers
 +============================================================}

{+------------------------------------------------------------
 | Procedure TTypedListgeneratorMainform.MacroGridDrawCell
 |
 | Event      : OnDrawCell
 | Used by    : MacroGrid
 | Call method: static
 | Visibility : published
 | Description:
 |   Draws the contents of the left column of the grid (the macro
 |   names) in boldface.
 | Error Conditions: none
 | Created: 03.06.99 by P. Below
 +------------------------------------------------------------}
Procedure TTypedListgeneratorMainform.MacroGridDrawCell(Sender: TObject;
    aCol, aRow: Integer; Rect: TRect; State: TGridDrawState);
  Begin
    If aCol = 0 Then Begin
      With (Sender As TStringGrid) Do Begin
        Canvas.Font.Style := [fsBold];
        Canvas.TextRect( Rect, Rect.Left+4, Rect.Top + 2, Cells[0, aRow]);
        Canvas.Font.Style := [];
      End; { With }
    End; { If }
  End; { TTypedListgeneratorMainform.MacroGridDrawCell }

{+============================================================
 | Section 2: Methods of TTypedListgeneratorMainform
 |    Part 5. Auxillary methods
 +============================================================}

{+------------------------------------------------------------
 | Procedure TTypedListgeneratorMainform.BuildUnit
 |
 | Parameters :
 |   unitfilename: full pathname of unitfile to generate
 | Call method: static
 | Visibility : private
 | Description:
 |   The method walks through the unit template line by line and
 |   replaces all macro strings found in the text by appropriate
 |   values. The generated lines are stored in a stringlist, which
 |   is finally written to a file with the passed name. The user
 |   is informed of the success of this operation.
 | Error Conditions:
 |   A EMacroError exception will be raised if the code finds a
 |   macro start delimiter but not a matching end delimiter in a
 |   line or if the passed unitfilename is empty.
 | Created: 03.06.99 by P. Below
 +------------------------------------------------------------}
Procedure TTypedListgeneratorMainform.BuildUnit(
    Const unitfilename: String );

  {+Level 2: Host TTypedListgeneratorMainform.BuildUnit
   | Replace all macros in the passed string with their vales.
   +}
  Procedure ReplaceMacros( Var S: String );
    Var
      n, k: Integer;
      macrostring: String;
    Begin 
      Repeat 
        { Find start delimiter of a macro. } 
        n:= Pos( '<!', S );
        If n > 0 Then Begin 
          { Found one, find end delimiter. }
          k:= IScan( '>', S, n+1 );
          If k > 0 Then Begin 
            { Found that, too. Copy macro from S, delete
              it and insert the macro value in its place. }
            Macrostring := Copy( S, n+2, k-n-2 );
            Delete( S, n, k-n+1 );
            Insert( Macro( macrostring ), S, n );
          End { If }
          Else
            raise EMacroError.CreateFmt(
                    eUnterminatedMacro, [S] );
        End; { If }
      Until n = 0;
    End; { ReplaceMacros } 
    
  Var
    unittext: TStringlist;
    i: Integer;
    S: String;
  Begin { TTypedListgeneratorMainform.BuildUnit }
    If Length( unitfilename ) = 0 Then
      raise EMacroError.Create( eUnitnameRequired );

    unittext := TStringlist.Create;
    Try 
      With Unittemplate.Lines Do Begin 
        For i := 0 To Count-1 Do Begin 
          S:= Strings[ i ];
          ReplaceMacros( S );
          unittext.Add( S );
        End; { For }
      End; { With }
      unittext.SaveToFile( unitfilename );
      MessageDlg( Format( 'Unit %s has been created in directory %s',
                          [ ExtractFilename( unitfilename ),
                            ExtractFileDir( unitfilename ) ] ),
                  mtInformation, [ mbOK ], 0 );
    Finally 
      unittext.Free;
    End; { Try Finally }
  End; { TTypedListgeneratorMainform.BuildUnit }

{+------------------------------------------------------------
 | Function  TTypedListgeneratorMainform.Macro
 |
 | Parameters :
 |   macroname: name of the macro to find
 | Returns    : value of this macro, may be emptyStr
 | Call method: static
 | Visibility : private
 | Description:
 |   The method searches for the passed macro name first in the 
 |   list of build-in macros and then in the macroGrid. 
 | Error Conditions: 
 |   If the macro is not found an EMacroError exception is raised.
 |   This is highly inlikely, however.
 | Created: 03.06.99 by P. Below
 +------------------------------------------------------------}
Function  TTypedListgeneratorMainform.Macro( 
    Const macroname: String ): String;
  Var
    i: Integer;
  Begin
    { First check the known macros }
    i:= FKnownMacros.IndexOf( macroname );
    If i >= 0 Then Begin 
      { This is a build-in macro, call its value function to
        obtain the value. See BuildMacroList below. }
      Result := TMacroProc( FKnownMacros.Objects[ i ] );
    End { If }
    Else Begin 
      { Scan the grid } 
      i:= MacroGrid.Cols[ 0 ].IndexOf( macroname );
      If i >= 0 Then 
        Result := Trim( MacroGrid.Cells[ 1, i ] )
      Else
        raise EMacroError.CreateFmt( eUnknownMacro, 
                                   [ macroname ] );
    End; { Else }
  End; { TTypedListgeneratorMainform.Macro }

{+------------------------------------------------------------
 | Procedure TTypedListgeneratorMainform.BuildMacroLists
 |
 | Parameters : none
 | Call method: static
 | Visibility : private
 | Description:
 |   The method first adds all the build-in macros together with
 |   their value functions to the FKnownMacros list. It then walks
 |   over the unit template line by line and adds any unknown macros
 |   found to the FunknownMacros list. This list is finally used
 |   to set the MacroGrid rowcount to the number of found macros
 |   and to display the macro names in the left column of the grid.
 | Error Conditions: 
 |   A EMacroError exception will be raised if the code finds a 
 |   macro start delimiter but not a matching end delimiter in a 
 |   line.
 | Created: 03.06.99 by P. Below
 +------------------------------------------------------------}
Procedure TTypedListgeneratorMainform.BuildMacroLists;

  {+Level 2: TTypedListgeneratorMainform.BuildMacroLists
   | Isolates all macros from the passed string and adds the macro
   | name to the list of unknown macros if it is not a known one.
   +}
  Procedure ExtractMacros( S : String );
    Var
      n: Integer;
      macro: String;
    Begin 
      Repeat 
        { Find macro start delimiter. }
        n := Pos( '<!', S );
        If n > 0 Then Begin 
          { Found one, delete it with all preceeding text from
            the string and search for the end delimiter. }
          Delete( S, 1, n+1 );
          n:= Pos( '>',S );
          If n > 0 Then Begin 
            { Copy the macro to a string, see if we already know it,
              if not, add it to FUnknownMacros. Then delete the
              macro from S. }
            macro:= Copy( S, 1, n-1 );
            If ( FKnownMacros.IndexOf( macro ) < 0 ) and
               ( FUnknownMacros.Indexof( macro ) < 0 ) 
            Then Begin 
              FUnknownMacros.Add( macro );
            End; { If }
            Delete( S, 1, n );
          End { If }
          Else
            raise EMacroError.CreateFmt(
                    eUnterminatedMacro, [S] );
        End; { If }
      Until n = 0;
    End; { ExtractMacros }

  Var
    i: Integer;
  Begin { TTypedListgeneratorMainform.BuildMacroLists }
    { Add the build-in macros. }
    With FKnownMacros Do Begin 
      AddObject( 'DATE',    TObject( @MacroDate ));
      AddObject( 'VERSION', TObject( @MacroVersion ));
      AddObject( 'DEFINES', TObject( @MacroDefines ));
      { Note: the build-in macros are implemented by value functions
        that return the macros value when called. This is the 
        best way to do it since the value of these macros is
        not known at compile-time, with the exception of VERSION. 
        The value functions must be of type TMacroProc. }
    End; { With }

    { Look for user-defined macros in the unit template. }
    With UnitTemplate.Lines Do Begin 
      For i := 0 To Count-1 Do 
        ExtractMacros( Strings[ i ] );
    End; { With }

    { Show the user-defined macros in the macro grid. }
    With MacroGrid Do Begin 
      RowCount := FUnknownMacros.Count;
      Cols[ 0 ].Assign( FUnknownMacros );
    End; { With }
  End; { TTypedListgeneratorMainform.BuildMacroLists }

{+------------------------------------------------------------
 | Procedure TTypedListgeneratorMainform.SizeGrid
 |
 | Parameters : none
 | Call method: static
 | Visibility : private
 | Description:
 |   Measure the display width of the macro names in column 0
 |   of the grid, size the column accordingly and size the 
 |   column 1 to take up most of the grids remainder.
 | Error Conditions: none
 | Created: 03.06.99 by P. Below
 +------------------------------------------------------------}
Procedure TTypedListgeneratorMainform.SizeGrid;
  Var
    max, i, w: Integer;
  Begin
    w:= 0;
    With MacroGrid Do Begin
      Canvas.Font := Font;
      Canvas.Font.Style := [fsBold];
      max := ColWidths[ 0 ];
      For i := 0 To RowCount-1 Do Begin
        w:= Canvas.TextWidth( Cells[ 0,i ] );
        If w > max Then
          max := w;
      End; { For }
      w:= w + 10;
      ColWidths[ 0 ] := w;
      ColWidths[ 1 ] := ClientWidth - w - 5;
    End; { With }
  End; { TTypedListgeneratorMainform.SizeGrid }

{+------------------------------------------------------------
 | Procedure TTypedListgeneratorMainform.EvaluateCommandline
 |
 | Parameters : none
 | Call method: static
 | Visibility : private
 | Description:
 |   The user can specify a template file to use on the commandline.
 |   If he does so we load this file into the Unittemplate text
 |   container. If the file does not exists we issue an error 
 |   and use the default template instead.
 | Error Conditions: none
 | Created: 03.06.99 by P. Below
 +------------------------------------------------------------}
Procedure TTypedListgeneratorMainform.EvaluateCommandline;
  Var
    filename: String;
  Begin
    If ParamCount > 0 Then Begin
      filename := ParamStr( 1 );
      If FileExists( filename ) Then 
        Unittemplate.Lines.Loadfromfile( filename )
      Else
        MessageDlg( 
          format( eFileNotFound, [filename] ),
          mtError, [mbOK], 0 )
    End; { If }
  End; { TTypedListgeneratorMainform.EvaluateCommandline }

{+------------------------------------------------------------
 | Procedure TTypedListgeneratorMainform.VerifyMacroInput
 |
 | Parameters : none
 | Call method: static
 | Visibility : private
 | Description:
 |   Checks through the MacroGrid for macros for which the user
 |   has not supplied a value. If any are found a message is 
 |   shown asking the user to confirm that he wants to go ahead
 |   anyway. 
 | Error Conditions: 
 |   Will raise a silent exception if the user does not accept
 |   empty macro values.
 | Created: 03.06.99 by P. Below
 +------------------------------------------------------------}
Procedure TTypedListgeneratorMainform.VerifyMacroInput;
  Var
    numEmptyMacros, i: Integer;
    Emptymacros, temp: String;

  {+Level 2: Host TTypedListgeneratorMainform.VerifyMacroInput}
  Procedure Add( Const S: String ); 
    Begin
      Inc( numEmptyMacros );
      If numEmptyMacros > 1 Then 
        Emptymacros:= Emptymacros + ', ';
      Emptymacros := Emptymacros + S;
    End; { Add }
    
  Begin { TTypedListgeneratorMainform.VerifyMacroInput }
    Emptymacros := EmptyStr;
    numEmptyMacros := 0;
    With MacroGrid Do Begin
      For i := 0 To RowCount-1 Do Begin
        If Length( Trim( Cells[1, i] )) = 0 Then 
          Add( Cells[0, i] );
      End; { For }
    End; { With }

    If numEmptyMacros > 0 Then Begin
      If numEmptyMacros = 1 Then
        temp := cOneEmptyMacro
      Else
        temp := cSeveralEmptyMacros;
      If MessageDlg(
           Format( temp, [Emptymacros] ),
           mtConfirmation,
           mbYesNoCancel,
           0 ) <> mrYes 
      Then 
        Abort;
    End; { If }
  End; { TTypedListgeneratorMainform.VerifyMacroInput }

End { GenTypedLists1 }.


